#include <gst/gst.h>
#include <gst/app/gstappsrc.h>

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include<gst/video/video.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

uint8_t b_white[645*485*3];
uint8_t b_black[645*485*3];

GST_DEBUG_CATEGORY (appsrc_pipeline_debug);
#define GST_CAT_DEFAULT appsrc_pipeline_debug

typedef struct _App App;

struct _App
{
	GstElement *pipeline;
	GstElement *appsrc;

	GMainLoop *loop;
	guint sourceid;

	GTimer *timer;

};

App s_app;
static gboolean read_data(App *app) {

	static gboolean white = FALSE;
	static GstClockTime timestamp = 0;
	GstBuffer *buffer;
	guint size;
	GstFlowReturn ret;

	size = 640 * 480 * 3;

	buffer = gst_buffer_new_wrapped_full( 0, (gpointer)(white?b_white:b_black), size, 0, size, NULL, NULL );

	white = !white;
	GST_BUFFER_PTS (buffer) = timestamp;
	GST_BUFFER_DURATION (buffer) = gst_util_uint64_scale_int (1, GST_SECOND, 4);
	timestamp += GST_BUFFER_DURATION (buffer);
	GST_BUFFER_TIMESTAMP(buffer)=timestamp; 

	g_signal_emit_by_name (app->appsrc, "push-buffer", buffer, &ret);
	gst_buffer_unref (buffer);

	if (ret != GST_FLOW_OK) {
		/* something wrong, stop pushing */
		printf("error\n");
		// g_main_loop_quit (loop);
	}
	return TRUE;
}

	static void
start_feed (GstElement * pipeline, guint size, App * app)
{
	if (app->sourceid == 0) {
		GST_DEBUG ("start feeding");
		app->sourceid = g_idle_add ((GSourceFunc) read_data, app);
	}
}

	static void
stop_feed (GstElement * pipeline, App * app)
{
	if (app->sourceid != 0) {
		GST_DEBUG ("stop feeding");
		g_source_remove (app->sourceid);
		app->sourceid = 0;
	}
}


	static gboolean
bus_message (GstBus * bus, GstMessage * message, App * app)
{
	GST_DEBUG ("got message %s",
			gst_message_type_get_name (GST_MESSAGE_TYPE (message)));

	switch (GST_MESSAGE_TYPE (message)) {
		case GST_MESSAGE_ERROR: {
									GError *err = NULL;
									gchar *dbg_info = NULL;

									gst_message_parse_error (message, &err, &dbg_info);
									g_printerr ("ERROR from element %s: %s\n",
											GST_OBJECT_NAME (message->src), err->message);
									g_printerr ("Debugging info: %s\n", (dbg_info) ? dbg_info : "none");
									g_error_free (err);
									g_free (dbg_info);
									g_main_loop_quit (app->loop);
									break;
								}
		case GST_MESSAGE_EOS:
								g_main_loop_quit (app->loop);
								break;
		default:
								break;
	}
	return TRUE;
}



	int
main (int argc, char *argv[])
{
	App *app = &s_app;
	GError *error = NULL;
	GstBus *bus;
	GstCaps *caps;
	GstStateChangeReturn ret ;
	int i=0;

//	for ( i = 0; i < 385*288; i++) 
//	{   
//		b_black[i] = 0;
//		b_white[i] = 0xFFFF;
//	}   
	for ( i = 0; i < 640*480*3; ) 
	{   
		b_black[i] = 0; //rand()%255;
		i++;
		b_white[i] = 255; //rand()%255;
		i++;
		b_black[i] = 0; //rand()%255;
		i++;
		b_white[i] = 255; //rand()%255;
		i++;
		b_black[i] = 0; //rand()%255;
		i++;
		b_white[i] = 255; //rand()%255;
		i++;
	}   
	gst_init (NULL, NULL);

	GST_DEBUG_CATEGORY_INIT (appsrc_pipeline_debug, "appsrc-pipeline", 0,
			"appsrc pipeline example");

	/* create a mainloop to get messages and to handle the idle handler that will
	 * * feed data to appsrc. */
	app->loop = g_main_loop_new (NULL, TRUE);
	app->timer = g_timer_new();

	app->pipeline = gst_parse_launch("appsrc name=mysource ! videoconvert ! vpe num-input-buffers=8 ! queue ! ducatimpeg4enc bitrate=4000 ! queue ! mpeg4videoparse !  rtpmp4vpay config-interval=1 pt=96 ! udpsink host=172.24.129.207 port=5000", NULL);



	g_assert (app->pipeline);

	bus = gst_pipeline_get_bus (GST_PIPELINE (app->pipeline));
	g_assert(bus);

	/* add watch for messages */
	gst_bus_add_watch (bus, (GstBusFunc) bus_message, app);

#if 1
	GstVideoInfo info;
	/* get the appsrc */
	app->appsrc = gst_bin_get_by_name (GST_BIN(app->pipeline), "mysource");
	g_assert(app->appsrc);
	g_assert(GST_IS_APP_SRC(app->appsrc));
	g_signal_connect (app->appsrc, "need-data", G_CALLBACK (start_feed), app);
	g_signal_connect (app->appsrc, "enough-data", G_CALLBACK (stop_feed), app);
#endif

	/* set the caps on the source */
#if 1
	caps = gst_caps_new_simple ("video/x-raw",
			"format",G_TYPE_STRING,"RGB",
			"bpp",G_TYPE_INT,24,
			"depth",G_TYPE_INT,24,
			"width", G_TYPE_INT, 640,
			"height", G_TYPE_INT, 480,
			"framerate", GST_TYPE_FRACTION, 30, 1,
			NULL);

	gst_app_src_set_caps(GST_APP_SRC(app->appsrc), caps);
	g_object_set (app->appsrc, "format", GST_FORMAT_TIME, NULL); 

#endif
	/* go to playing and wait in a mainloop. */
	ret = gst_element_set_state (app->pipeline, GST_STATE_PLAYING); 
	if (ret == GST_STATE_CHANGE_FAILURE) {
		g_printerr ("Unable to set the pipeline to the playing state.\n");
		gst_object_unref (app->pipeline);
		return -1;
	}
	/* this mainloop is stopped when we receive an error or EOS */
	g_main_loop_run (app->loop);

	GST_DEBUG ("stopping");

	gst_element_set_state (app->pipeline, GST_STATE_NULL);

	gst_object_unref (bus);
	g_main_loop_unref (app->loop);

	return 0;
}





